%%%=========================================================
%%% OBJECTIVE: Provide MDP solutions for Bio and ES strategies
%%%========================================================


nb_node=8; % only 8 nodes,don't care about the breeding node
S=2^nb_node;

%%get all 2^8 possible habitat states
state_all=zeros(S,nb_node);
for i=1:S
state_all(i,:)=getState(i-1,nb_node);
end
state_all=state_all+1; % move 0,1,2, to 1,2,3 corresponding to 100%-80% good,80%-48% threatened, 40%-0% vulnerable

%% population network matrix
output=get_population(38000,nb_node);
M1=output(:,1:8); % get the population matrix
cc=output(:,9); % carrying capacity
%% action a--protect one habitat at each time step
a_eff=0.8;

%% habitat loss rate matrix for each node, upon changes order 26185743. for 30 years
hlr=[0.148725673 0.191413881 0.034216975 0.001 0.001 0.130491183 0.001 0.158600474];
%

%% get multiple action matrix
cost=0.01*cc'; % assume the cost is proportional to the carrying capacity %[100 80 20 60 120 20 70 60];
budget=400;
[action, nodeID]=getAction(cost,budget,nb_node);

%% transition probability between habitat states for habitat i
h = waitbar(0,'Please wait...');
A=size(action,1); % A=75 possible combination of actions
t=0;steps=S*S*A;
P={}; % construct general transition probability matrix
 for a=1:A
     PP=sparse(S,S);
     for s1=S:-1:1
         state1=getState(s1-1,nb_node); % get all possible state and set as in time t
         for s2=S:-1:1
             waitbar(t / steps)
             state2=getState(s2-1,nb_node); % get all possible state and set as in time t+1
             Pjt=getIndi_p2(state1,state2,a,a_eff,hlr,nodeID); % get individual transition probability from t ot t+1 given action a
             PP(s1,s2)=prod(Pjt);  % the general transition probability is the product of individual probabilities
             t=t+1;
         end
     end
     P{a}=PP;
 end
 close (h)


%%====================
%% Bio strategy
%%====================
 %%Reward matrix
[R_Bio,popback] =  getReward_Bio_multiaction(38000,nb_node,state_all,action,cost);%
% put large negative value when protecting non-existence node
state=state_all-1;
for i=1:S
    for a=1:size(action,1)
        
        state1=state(i,:);
        if ( any( state1(nodeID{a})==0 ))% any node under protection is absent
            R_Bio(i,a)=-100000;
        end
    end
end
R_Bio(1,:)=0;

%% MDP 
[V, policy_Bio, iter, cpu_time] = mdp_policy_iteration(P, R_Bio, 0.96);  % V_star=85.53 (overall time steps), policy iteration result=value iteration result

%% visualization 
ttt2=find(popback>0); % delete when theres 0 population back (inefficient protection)
policynew_Bio=policy_Bio(ttt2,:); % only keep the policy that actually works (not work means still protect a habitat when there's no population back-->popback=0 )

f_Bio=tabulate(policynew_Bio);f_Bio(~any(f_Bio(:,2:3),2),:)=[]; % protection frequency

node_f_Bio={}; % multi node protection corresponds to bar graph
for i=1:size(f_Bio,1)
     node_f_Bio{i}=cell2mat(nodeID(f_Bio(i,1)));
 end
% save('Bio_strategy')

%%====================================
%%  ES strategy
%%====================================
ES=[11069148573 5534574287 154061624.6 339211305.5 23481830.42 4912090161 47500000000 5534574287];
R_ES = getReward_ES(ES,action,cost );
% put large negative reward when protecting non-existence node
state=state_all-1;
for i=1:S
    for a=1:size(action,1)
        
        state1=state(i,:);
        if ( any( state1(nodeID{a})==0 ))% any node under protection is absent
            R_ES(i,a)=-100000;
        end
    end
end
R_ES(1,:)=0;

%MDP
[V, policy_ES, iter, cpu_time] = mdp_policy_iteration(P, R_ES, 0.96);  % V_star=85.53 (overall time steps), policy iteration result=value iteration result

policynew_ES=policy_ES(ttt2,:); % only keep the policy that actually works (not work means still protect a habitat when there's no population back-->popback=0 )

f_ES=tabulate(policynew_ES);f_ES(~any(f_ES(:,2:3),2),:)=[]; % protection frequency
 node_f_ES={}; % multi node protection corresponds to bar graph
 for i=1:size(f_ES,1)
     node_f_ES{i}=cell2mat(nodeID(f_ES(i,1)));
 end
% save('ES_strategy')


